@@ -40,49 +40,6 @@
 /* Attribute to create detached thread */
 extern pthread_attr_t th_attr_detached;
 
-static int unlink_mount_tree(struct autofs_point *ap, struct mnt_list *mnts)
-{
-	struct mnt_list *this;
-	int rv, ret;
-	pid_t pgrp = getpgrp();
-	char spgrp[20];
-
-	sprintf(spgrp, "pgrp=%d", pgrp);
-
-	ret = 1;
-	this = mnts;
-	while (this) {
-		if (strstr(this->opts, spgrp)) {
-			this = this->next;
-			continue;
-		}
-
-		if (strcmp(this->fs_type, "autofs"))
-			rv = spawn_umount(ap->logopt, "-l", this->path, NULL);
-		else
-			rv = umount2(this->path, MNT_DETACH);
-		if (rv == -1) {
-			debug(ap->logopt,
-			      "can't unlink %s from mount tree", this->path);
-
-			switch (errno) {
-			case EINVAL:
-				warn(ap->logopt,
-				      "bad superblock or not mounted");
-				break;
-
-			case ENOENT:
-			case EFAULT:
-				ret = 0;
-				warn(ap->logopt, "bad path for mount");
-				break;
-			}
-		}
-		this = this->next;
-	}
-	return ret;
-}
-
 static int do_mount_autofs_indirect(struct autofs_point *ap, const char *root)
 {
 	const char *str_indirect = mount_type_str(t_indirect);
@@ -93,7 +50,6 @@ static int do_mount_autofs_indirect(struct autofs_point *ap, const char *root)
 	const char *map_name = hosts_map_name;
 	const char *type;
 	struct stat st;
-	struct mnt_list *mnts;
 	int ret;
 	int err;
 
@@ -113,16 +69,12 @@ static int do_mount_autofs_indirect(struct autofs_point *ap, const char *root)
 		if (ret == 0)
 			return -1;
 	} else {
-		mnts = get_mnt_list(_PROC_MOUNTS, ap->path, 1);
-		if (mnts) {
-			ret = unlink_mount_tree(ap, mnts);
-			free_mnt_list(mnts);
-			if (!ret) {
-				error(ap->logopt,
-				      "already mounted as other than autofs "
-				      "or failed to unlink entry in tree");
-				goto out_err;
-			}
+		ret = unlink_mount_tree(ap, ap->path);
+		if (!ret) {
+			error(ap->logopt,
+			      "already mounted as other than autofs "
+			      "or failed to unlink entry in tree");
+			goto out_err;
 		}
 	}
 
@@ -132,6 +84,26 @@ static int do_mount_autofs_indirect(struct autofs_point *ap, const char *root)
 		goto out_err;
 	}
 
+	if ((ap->flags & MOUNT_FLAG_STRICTEXPIRE) &&
+	    ((get_kver_major() == 5 && get_kver_minor() > 3) ||
+	     (get_kver_major() > 5))) {
+		char *tmp = realloc(options, strlen(options) + 12);
+		if (tmp) {
+			strcat(tmp, ",strictexpire");
+			options = tmp;
+		}
+	}
+
+	if ((ap->flags & MOUNT_FLAG_IGNORE) &&
+	    ((get_kver_major() == 5 && get_kver_minor() > 4) ||
+	     (get_kver_major() > 5))) {
+		char *tmp = realloc(options, strlen(options) + 7);
+		if (tmp) {
+			strcat(tmp, ",ignore");
+			options = tmp;
+		}
+	}
+
 	/* In case the directory doesn't exist, try to mkdir it */
 	if (mkdir_path(root, mp_mode) < 0) {
 		if (errno != EEXIST && errno != EROFS) {
@@ -422,34 +394,34 @@ void *expire_proc_indirect(void *arg)
 	left = 0;
 
 	/* Get a list of real mounts and expire them if possible */
-	mnts = get_mnt_list(_PROC_MOUNTS, ap->path, 0);
+	mnts = get_mnt_list(ap->path, 0);
 	pthread_cleanup_push(mnts_cleanup, mnts);
 	for (next = mnts; next; next = next->next) {
 		char *ind_key;
 		int ret;
 
-		if (!strcmp(next->fs_type, "autofs")) {
+		if (next->flags & MNTS_AUTOFS) {
 			/*
 			 * If we have submounts check if this path lives below
 			 * one of them and pass on the state change.
 			 */
 			pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &cur_state);
-			if (strstr(next->opts, "indirect"))
-				master_notify_submount(ap, next->path, ap->state);
-			else if (strstr(next->opts, "offset")) {
+			if (next->flags & MNTS_INDIRECT)
+				master_notify_submount(ap, next->mp, ap->state);
+			else if (next->flags & MNTS_OFFSET) {
 				struct map_source *map;
 				struct mapent_cache *mc = NULL;
 				struct mapent *me = NULL;
 				struct stat st;
 
 				/* It's got a mount, deal with in the outer loop */
-				if (is_mounted(_PATH_MOUNTED, next->path, MNTS_REAL)) {
+				if (is_mounted(next->mp, MNTS_REAL)) {
 					pthread_setcancelstate(cur_state, NULL);
 					continue;
 				}
 
 				/* Don't touch submounts */
-				if (master_find_submount(ap, next->path)) {
+				if (master_find_submount(ap, next->mp)) {
 					pthread_setcancelstate(cur_state, NULL);
 					continue;
 				}
@@ -460,7 +432,7 @@ void *expire_proc_indirect(void *arg)
 				while (map) {
 					mc = map->mc;
 					cache_writelock(mc);
-					me = cache_lookup_distinct(mc, next->path);
+					me = cache_lookup_distinct(mc, next->mp);
 					if (me)
 						break;
 					cache_unlock(mc);
@@ -502,7 +474,7 @@ void *expire_proc_indirect(void *arg)
 		 * If the mount corresponds to an offset trigger then
 		 * the key is the path, otherwise it's the last component.
 		 */
-		ind_key = strrchr(next->path, '/');
+		ind_key = strrchr(next->mp, '/');
 		if (ind_key)
 			ind_key++;
 
@@ -515,7 +487,7 @@ void *expire_proc_indirect(void *arg)
 		 */
 		pthread_cleanup_push(master_source_lock_cleanup, ap->entry);
 		master_source_readlock(ap->entry);
-		me = lookup_source_mapent(ap, next->path, LKP_DISTINCT);
+		me = lookup_source_mapent(ap, next->mp, LKP_DISTINCT);
 		if (!me && ind_key)
 			me = lookup_source_mapent(ap, ind_key, LKP_NORMAL);
 		pthread_cleanup_pop(1);
@@ -527,10 +499,10 @@ void *expire_proc_indirect(void *arg)
 			cache_unlock(me->mc);
 		}
 
-		debug(ap->logopt, "expire %s", next->path);
+		debug(ap->logopt, "expire %s", next->mp);
 
 		pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &cur_state);
-		ret = ops->expire(ap->logopt, ioctlfd, next->path, how);
+		ret = ops->expire(ap->logopt, ioctlfd, next->mp, how);
 		if (ret)
 			left++;
 		pthread_setcancelstate(cur_state, NULL);
@@ -552,14 +524,14 @@ void *expire_proc_indirect(void *arg)
 	pthread_cleanup_pop(1);
 
 	count = offsets = submnts = 0;
-	mnts = get_mnt_list(_PROC_MOUNTS, ap->path, 0);
+	mnts = get_mnt_list(ap->path, 0);
 	pthread_cleanup_push(mnts_cleanup, mnts);
 	/* Are there any real mounts left */
 	for (next = mnts; next; next = next->next) {
-		if (strcmp(next->fs_type, "autofs"))
+		if (!(next->flags & MNTS_AUTOFS))
 			count++;
 		else {
-			if (strstr(next->opts, "indirect"))
+			if (next->flags & MNTS_INDIRECT)
 				submnts++;
 			else
 				offsets++;
@@ -568,7 +540,7 @@ void *expire_proc_indirect(void *arg)
 	pthread_cleanup_pop(1);
 
 	if (submnts)
-		info(ap->logopt,
+		debug(ap->logopt,
 		     "%d submounts remaining in %s", submnts, ap->path);
 
 	/* 
@@ -576,7 +548,7 @@ void *expire_proc_indirect(void *arg)
 	 * words) the umounts are done by the time we reach here
 	 */
 	if (count)
-		info(ap->logopt, "%d remaining in %s", count, ap->path);
+		debug(ap->logopt, "%d remaining in %s", count, ap->path);
 
 	ec.status = left;
 
@@ -748,6 +720,12 @@ static void *do_mount_indirect(void *arg)
 
 	pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &state);
 
+	if (defaults_get_mount_verbose()) {
+		pid_t ppid = log_pidinfo(ap, mt.pid, "requestor");
+		if (ppid > 0)
+			log_pidinfo(ap, ppid, "parent");
+	}
+
 	len = ncat_path(buf, sizeof(buf), ap->path, mt.name, mt.len);
 	if (!len) {
 		crit(ap->logopt, "path to be mounted is to long");
