@@ -786,10 +786,163 @@ done:
 	return ret;
 }
 
+/* From glibc decode_name() */
+/* Since the values in a line are separated by spaces, a name cannot
+ * contain a space.  Therefore some programs encode spaces in names
+ * by the strings "\040".  We undo the encoding when reading an entry.
+ * The decoding happens in place.
+ */
+static char *local_decode_name(char *buf)
+{
+	char *rp = buf;
+	char *wp = buf;
+
+	do {
+		if (rp[0] == '\\' && rp[1] == '0' &&
+		    rp[2] == '4' && rp[3] == '0') {
+			/* \040 is a SPACE.  */
+			*wp++ = ' ';
+			rp += 3;
+		} else if (rp[0] == '\\' && rp[1] == '0' &&
+			   rp[2] == '1' && rp[3] == '1') {
+			/* \011 is a TAB.  */
+			*wp++ = '\t';
+			rp += 3;
+		} else if (rp[0] == '\\' && rp[1] == '0' &&
+			   rp[2] == '1' && rp[3] == '2') {
+			/* \012 is a NEWLINE.  */
+			*wp++ = '\n';
+			rp += 3;
+		} else if (rp[0] == '\\' && rp[1] == '\\') {
+			/*
+			 * We have to escape \\ to be able to represent
+			 * all characters.
+			 */
+			*wp++ = '\\';
+			rp += 1;
+		} else if (rp[0] == '\\' && rp[1] == '1' &&
+			   rp[2] == '3' && rp[3] == '4') {
+			/* \134 is also \\.  */
+			*wp++ = '\\';
+			rp += 3;
+		} else
+			*wp++ = *rp;
+	} while (*rp++ != '\0');
+
+	return buf;
+}
+
+/* From glibc getmntent_r() */
+static struct mntent *
+local_getmntent_r(FILE *tab, struct mntent *mnt, char *buf, int size)
+{
+	char *cp, *head;
+
+	do {
+		char *end_ptr;
+
+		if (fgets(buf, size, tab) == NULL)
+			return 0;
+
+		end_ptr = strchr(buf, '\n');
+		if (end_ptr != NULL) {
+			while (end_ptr[-1] == ' ' || end_ptr[-1] == '\t')
+				end_ptr--;
+			*end_ptr = '\0';
+		} else {
+			/* Whole line was not read. Do it now but forget it. */
+			char tmp[1024];
+			while (fgets(tmp, sizeof tmp, tab) != NULL)
+				if (strchr(tmp, '\n') != NULL)
+					break;
+		}
+
+		head = buf + strspn(buf, " \t");
+	/* skip empty lines and comment lines */
+	} while (head[0] == '\0' || head[0] == '#');
+
+	cp = strsep(&head, " \t");
+	mnt->mnt_fsname = cp != NULL ? local_decode_name(cp) : (char *) "";
+	if (head)
+		head += strspn(head, " \t");
+	cp = strsep(&head, " \t");
+	mnt->mnt_dir = cp != NULL ? local_decode_name (cp) : (char *) "";
+	if (head)
+		head += strspn(head, " \t");
+	cp = strsep(&head, " \t");
+	mnt->mnt_type = cp != NULL ? local_decode_name (cp) : (char *) "";
+	if (head)
+		head += strspn (head, " \t");
+	cp = strsep (&head, " \t");
+	mnt->mnt_opts = cp != NULL ? local_decode_name (cp) : (char *) "";
+
+	/* autofs doesn't need freq or passno */
+
+	return mnt;
+}
+
+int unlink_mount_tree(struct autofs_point *ap, const char *mp)
+{
+	FILE *tab;
+	struct mntent *mnt;
+	struct mntent mnt_wrk;
+	char buf[PATH_MAX * 3];
+	unsigned int mp_len = strlen(mp);
+	int rv, ret = 1;
+
+	tab = open_fopen_r(_PROC_MOUNTS);
+	if (!tab) {
+		char *estr = strerror_r(errno, buf, PATH_MAX - 1);
+		logerr("fopen: %s", estr);
+		return 0;
+	}
+
+	while ((mnt = local_getmntent_r(tab, &mnt_wrk, buf, PATH_MAX * 3))) {
+		unsigned int mnt_dir_len;
+		int is_autofs;
+
+		if (strncmp(mnt->mnt_dir, mp, mp_len))
+			continue;
+
+		mnt_dir_len = strlen(mnt->mnt_dir);
+		is_autofs = !strcmp(mnt->mnt_type, "autofs");
+
+		if (mnt_dir_len == mp_len && !is_autofs) {
+			ret = 0;
+			break;
+		}
+
+		if (is_autofs)
+			rv = umount2(mnt->mnt_dir, MNT_DETACH);
+		else
+			rv = spawn_umount(ap->logopt, "-l", mnt->mnt_dir, NULL);
+		if (rv == -1) {
+			debug(ap->logopt,
+			      "can't unlink %s from mount tree", mnt->mnt_dir);
+
+			switch (errno) {
+			case EINVAL:
+				warn(ap->logopt,
+				      "bad superblock or not mounted");
+				break;
+
+			case ENOENT:
+			case EFAULT:
+				ret = 0;
+				warn(ap->logopt, "bad path for mount");
+				break;
+			}
+		}
+	}
+	fclose(tab);
+
+	return ret;
+}
+
 /*
  * Get list of mounts under path in longest->shortest order
  */
-struct mnt_list *get_mnt_list(const char *table, const char *path, int include)
+struct mnt_list *get_mnt_list(const char *path, int include)
 {
 	FILE *tab;
 	size_t pathlen = strlen(path);
@@ -798,28 +951,27 @@ struct mnt_list *get_mnt_list(const char *table, const char *path, int include)
 	struct mntent *mnt;
 	struct mnt_list *ent, *mptr, *last;
 	struct mnt_list *list = NULL;
-	char *pgrp;
 	size_t len;
 
 	if (!path || !pathlen || pathlen > PATH_MAX)
 		return NULL;
 
-	tab = open_setmntent_r(table);
+	tab = open_fopen_r(_PROC_MOUNTS);
 	if (!tab) {
 		char *estr = strerror_r(errno, buf, PATH_MAX - 1);
-		logerr("setmntent: %s", estr);
+		logerr("fopen: %s", estr);
 		return NULL;
 	}
 
-	while ((mnt = getmntent_r(tab, &mnt_wrk, buf, PATH_MAX * 3))) {
+	while ((mnt = local_getmntent_r(tab, &mnt_wrk, buf, PATH_MAX * 3))) {
 		len = strlen(mnt->mnt_dir);
 
 		if ((!include && len <= pathlen) ||
 	  	     strncmp(mnt->mnt_dir, path, pathlen) != 0)
 			continue;
 
-		/* Not a subdirectory of requested path ? */
-		/* pathlen == 1 => everything is subdir    */
+		/* Not a subdirectory of requested mp? */
+		/* mp_len == 1 => everything is subdir    */
 		if (pathlen > 1 && len > pathlen &&
 				mnt->mnt_dir[pathlen] != '/')
 			continue;
@@ -835,7 +987,7 @@ struct mnt_list *get_mnt_list(const char *table, const char *path, int include)
 		mptr = list;
 		last = NULL;
 		while (mptr) {
-			if (len >= strlen(mptr->path))
+			if (len >= strlen(mptr->mp))
 				break;
 			last = mptr;
 			mptr = mptr->next;
@@ -848,73 +1000,31 @@ struct mnt_list *get_mnt_list(const char *table, const char *path, int include)
 
 		ent->next = mptr;
 
-		ent->path = malloc(len + 1);
-		if (!ent->path) {
+		ent->mp = malloc(len + 1);
+		if (!ent->mp) {
 			endmntent(tab);
 			free_mnt_list(list);
 			return NULL;
 		}
-		strcpy(ent->path, mnt->mnt_dir);
+		strcpy(ent->mp, mnt->mnt_dir);
 
-		ent->fs_name = malloc(strlen(mnt->mnt_fsname) + 1);
-		if (!ent->fs_name) {
-			endmntent(tab);
-			free_mnt_list(list);
-			return NULL;
-		}
-		strcpy(ent->fs_name, mnt->mnt_fsname);
+		if (!strcmp(mnt->mnt_type, "autofs"))
+			ent->flags |= MNTS_AUTOFS;
 
-		ent->fs_type = malloc(strlen(mnt->mnt_type) + 1);
-		if (!ent->fs_type) {
-			endmntent(tab);
-			free_mnt_list(list);
-			return NULL;
-		}
-		strcpy(ent->fs_type, mnt->mnt_type);
-
-		ent->opts = malloc(strlen(mnt->mnt_opts) + 1);
-		if (!ent->opts) {
-			endmntent(tab);
-			free_mnt_list(list);
-			return NULL;
-		}
-		strcpy(ent->opts, mnt->mnt_opts);
-
-		ent->owner = 0;
-		pgrp = strstr(mnt->mnt_opts, "pgrp=");
-		if (pgrp) {
-			char *end = strchr(pgrp, ',');
-			if (end)
-				*end = '\0';
-			sscanf(pgrp, "pgrp=%d", &ent->owner);
+		if (ent->flags & MNTS_AUTOFS) {
+			if (strstr(mnt->mnt_opts, "indirect"))
+				ent->flags |= MNTS_INDIRECT;
+			else if (strstr(mnt->mnt_opts, "direct"))
+				ent->flags |= MNTS_DIRECT;
+			else if (strstr(mnt->mnt_opts, "offset"))
+				ent->flags |= MNTS_OFFSET;
 		}
 	}
-	endmntent(tab);
+	fclose(tab);
 
 	return list;
 }
 
-/*
- * Reverse a list of mounts
- */
-struct mnt_list *reverse_mnt_list(struct mnt_list *list)
-{
-	struct mnt_list *next, *last;
-
-	if (!list)
-		return NULL;
-
-	next = list;
-	last = NULL;
-	while (next) {
-		struct mnt_list *this = next;
-		next = this->next;
-		this->next = last;
-		last = this;
-	}
-	return last;
-}
-
 void free_mnt_list(struct mnt_list *list)
 {
 	struct mnt_list *next;
@@ -928,42 +1038,33 @@ void free_mnt_list(struct mnt_list *list)
 
 		next = this->next;
 
-		if (this->path)
-			free(this->path);
-
-		if (this->fs_name)
-			free(this->fs_name);
-
-		if (this->fs_type)
-			free(this->fs_type);
-
-		if (this->opts)
-			free(this->opts);
+		if (this->mp)
+			free(this->mp);
 
 		free(this);
 	}
 }
 
-static int table_is_mounted(const char *table, const char *path, unsigned int type)
+static int table_is_mounted(const char *mp, unsigned int type)
 {
 	struct mntent *mnt;
 	struct mntent mnt_wrk;
 	char buf[PATH_MAX * 3];
-	size_t pathlen = strlen(path);
+	size_t mp_len = strlen(mp);
 	FILE *tab;
 	int ret = 0;
 
-	if (!path || !pathlen || pathlen >= PATH_MAX)
+	if (!mp || !mp_len || mp_len >= PATH_MAX)
 		return 0;
 
-	tab = open_setmntent_r(table);
+	tab = open_fopen_r(_PROC_MOUNTS);
 	if (!tab) {
 		char *estr = strerror_r(errno, buf, PATH_MAX - 1);
-		logerr("setmntent: %s", estr);
+		logerr("fopen: %s", estr);
 		return 0;
 	}
 
-	while ((mnt = getmntent_r(tab, &mnt_wrk, buf, PATH_MAX * 3))) {
+	while ((mnt = local_getmntent_r(tab, &mnt_wrk, buf, PATH_MAX * 3))) {
 		size_t len = strlen(mnt->mnt_dir);
 
 		if (type) {
@@ -980,17 +1081,17 @@ static int table_is_mounted(const char *table, const char *path, unsigned int ty
 					continue;
 		}
 
-		if (pathlen == len && !strncmp(path, mnt->mnt_dir, pathlen)) {
+		if (mp_len == len && !strncmp(mp, mnt->mnt_dir, mp_len)) {
 			ret = 1;
 			break;
 		}
 	}
-	endmntent(tab);
+	fclose(tab);
 
 	return ret;
 }
 
-static int ioctl_is_mounted(const char *table, const char *path, unsigned int type)
+static int ioctl_is_mounted(const char *mp, unsigned int type)
 {
 	struct ioctl_ops *ops = get_ioctl_ops();
 	unsigned int mounted;
@@ -999,9 +1100,9 @@ static int ioctl_is_mounted(const char *table, const char *path, unsigned int ty
 	/* If the ioctl fails fall back to the potentially resource
 	 * intensive mount table check.
 	 */
-	ret = ops->ismountpoint(LOGOPT_NONE, -1, path, &mounted);
+	ret = ops->ismountpoint(LOGOPT_NONE, -1, mp, &mounted);
 	if (ret == -1)
-		return table_is_mounted(table, path, type);
+		return table_is_mounted(mp, type);
 
 	if (mounted) {
 		switch (type) {
@@ -1016,43 +1117,14 @@ static int ioctl_is_mounted(const char *table, const char *path, unsigned int ty
 	return 0;
 }
 
-int is_mounted(const char *table, const char *path, unsigned int type)
+int is_mounted(const char *mp, unsigned int type)
 {
 	struct ioctl_ops *ops = get_ioctl_ops();
 
 	if (ops->ismountpoint)
-		return ioctl_is_mounted(table, path, type);
+		return ioctl_is_mounted(mp, type);
 	else
-		return table_is_mounted(table, path, type);
-}
-
-int has_fstab_option(const char *opt)
-{
-	struct mntent *mnt;
-	struct mntent mnt_wrk;
-	char buf[PATH_MAX * 3];
-	FILE *tab;
-	int ret = 0;
-
-	if (!opt)
-		return 0;
-
-	tab = open_setmntent_r(_PATH_MNTTAB);
-	if (!tab) {
-		char *estr = strerror_r(errno, buf, PATH_MAX - 1);
-		logerr("setmntent: %s", estr);
-		return 0;
-	}
-
-	while ((mnt = getmntent_r(tab, &mnt_wrk, buf, PATH_MAX * 3))) {
-		if (hasmntopt(mnt, opt)) {
-			ret = 1;
-			break;
-		}
-	}
-	endmntent(tab);
-
-	return ret;
+		return table_is_mounted(mp, type);
 }
 
 /*
@@ -1087,30 +1159,19 @@ void tree_free_mnt_tree(struct mnt_list *tree)
 
 		list_del(&this->self);
 
-		free(this->path);
-		free(this->fs_name);
-		free(this->fs_type);
-
-		if (this->opts)
-			free(this->opts);
+		free(this->mp);
 
 		free(this);
 	}
 
-	free(tree->path);
-	free(tree->fs_name);
-	free(tree->fs_type);
-
-	if (tree->opts)
-		free(tree->opts);
-
+	free(tree->mp);
 	free(tree);
 }
 
 /*
  * Make tree of system mounts in /proc/mounts.
  */
-struct mnt_list *tree_make_mnt_tree(const char *table, const char *path)
+struct mnt_list *tree_make_mnt_tree(const char *path)
 {
 	FILE *tab;
 	struct mntent mnt_wrk;
@@ -1118,20 +1179,19 @@ struct mnt_list *tree_make_mnt_tree(const char *table, const char *path)
 	struct mntent *mnt;
 	struct mnt_list *ent, *mptr;
 	struct mnt_list *tree = NULL;
-	char *pgrp;
 	size_t plen;
 	int eq;
 
-	tab = open_setmntent_r(table);
+	tab = open_fopen_r(_PROC_MOUNTS);
 	if (!tab) {
 		char *estr = strerror_r(errno, buf, PATH_MAX - 1);
-		logerr("setmntent: %s", estr);
+		logerr("fopen: %s", estr);
 		return NULL;
 	}
 
 	plen = strlen(path);
 
-	while ((mnt = getmntent_r(tab, &mnt_wrk, buf, PATH_MAX * 3))) {
+	while ((mnt = local_getmntent_r(tab, &mnt_wrk, buf, PATH_MAX * 3))) {
 		size_t len = strlen(mnt->mnt_dir);
 
 		/* Not matching path */
@@ -1155,61 +1215,31 @@ struct mnt_list *tree_make_mnt_tree(const char *table, const char *path)
 		INIT_LIST_HEAD(&ent->entries);
 		INIT_LIST_HEAD(&ent->sublist);
 
-		ent->path = malloc(len + 1);
-		if (!ent->path) {
+		ent->mp = malloc(len + 1);
+		if (!ent->mp) {
 			endmntent(tab);
 			free(ent);
 			tree_free_mnt_tree(tree);
 			return NULL;
 		}
-		strcpy(ent->path, mnt->mnt_dir);
+		strcpy(ent->mp, mnt->mnt_dir);
 
-		ent->fs_name = malloc(strlen(mnt->mnt_fsname) + 1);
-		if (!ent->fs_name) {
-			free(ent->path);
-			free(ent);
-			endmntent(tab);
-			tree_free_mnt_tree(tree);
-			return NULL;
-		}
-		strcpy(ent->fs_name, mnt->mnt_fsname);
+		if (!strcmp(mnt->mnt_type, "autofs"))
+			ent->flags |= MNTS_AUTOFS;
 
-		ent->fs_type = malloc(strlen(mnt->mnt_type) + 1);
-		if (!ent->fs_type) {
-			free(ent->fs_name);
-			free(ent->path);
-			free(ent);
-			endmntent(tab);
-			tree_free_mnt_tree(tree);
-			return NULL;
-		}
-		strcpy(ent->fs_type, mnt->mnt_type);
-
-		ent->opts = malloc(strlen(mnt->mnt_opts) + 1);
-		if (!ent->opts) {
-			free(ent->fs_type);
-			free(ent->fs_name);
-			free(ent->path);
-			free(ent);
-			endmntent(tab);
-			tree_free_mnt_tree(tree);
-			return NULL;
-		}
-		strcpy(ent->opts, mnt->mnt_opts);
-
-		ent->owner = 0;
-		pgrp = strstr(mnt->mnt_opts, "pgrp=");
-		if (pgrp) {
-			char *end = strchr(pgrp, ',');
-			if (end)
-				*end = '\0';
-			sscanf(pgrp, "pgrp=%d", &ent->owner);
+		if (ent->flags & MNTS_AUTOFS) {
+			if (strstr(mnt->mnt_opts, "indirect"))
+				ent->flags |= MNTS_INDIRECT;
+			else if (strstr(mnt->mnt_opts, "direct"))
+				ent->flags |= MNTS_DIRECT;
+			else if (strstr(mnt->mnt_opts, "offset"))
+				ent->flags |= MNTS_OFFSET;
 		}
 
 		mptr = tree;
 		while (mptr) {
-			int elen = strlen(ent->path);
-			int mlen = strlen(mptr->path);
+			int elen = strlen(ent->mp);
+			int mlen = strlen(mptr->mp);
 
 			if (elen < mlen) {
 				if (mptr->left) {
@@ -1229,7 +1259,7 @@ struct mnt_list *tree_make_mnt_tree(const char *table, const char *path)
 				}
 			}
 
-			eq = strcmp(ent->path, mptr->path);
+			eq = strcmp(ent->mp, mptr->mp);
 			if (eq < 0) {
 				if (mptr->left)
 					mptr = mptr->left;
@@ -1253,7 +1283,7 @@ struct mnt_list *tree_make_mnt_tree(const char *table, const char *path)
 		if (!tree)
 			tree = ent;
 	}
-	endmntent(tab);
+	fclose(tab);
 
 	return tree;
 }
@@ -1269,7 +1299,7 @@ int tree_get_mnt_list(struct mnt_list *mnts, struct list_head *list, const char
 		return 0;
 
 	plen = strlen(path);
-	mlen = strlen(mnts->path);
+	mlen = strlen(mnts->mp);
 	if (mlen < plen)
 		return tree_get_mnt_list(mnts->right, list, path, include);
 	else {
@@ -1278,10 +1308,10 @@ int tree_get_mnt_list(struct mnt_list *mnts, struct list_head *list, const char
 		tree_get_mnt_list(mnts->left, list, path, include);
 
 		if ((!include && mlen <= plen) ||
-				strncmp(mnts->path, path, plen))
+				strncmp(mnts->mp, path, plen))
 			goto skip;
 
-		if (plen > 1 && mlen > plen && mnts->path[plen] != '/')
+		if (plen > 1 && mlen > plen && mnts->mp[plen] != '/')
 			goto skip;
 
 		INIT_LIST_HEAD(&mnts->list);
@@ -1316,7 +1346,7 @@ int tree_get_mnt_sublist(struct mnt_list *mnts, struct list_head *list, const ch
 		return 0;
 
 	plen = strlen(path);
-	mlen = strlen(mnts->path);
+	mlen = strlen(mnts->mp);
 	if (mlen < plen)
 		return tree_get_mnt_sublist(mnts->right, list, path, include);
 	else {
@@ -1325,10 +1355,10 @@ int tree_get_mnt_sublist(struct mnt_list *mnts, struct list_head *list, const ch
 		tree_get_mnt_sublist(mnts->left, list, path, include);
 
 		if ((!include && mlen <= plen) ||
-				strncmp(mnts->path, path, plen))
+				strncmp(mnts->mp, path, plen))
 			goto skip;
 
-		if (plen > 1 && mlen > plen && mnts->path[plen] != '/')
+		if (plen > 1 && mlen > plen && mnts->mp[plen] != '/')
 			goto skip;
 
 		INIT_LIST_HEAD(&mnts->sublist);
@@ -1360,7 +1390,7 @@ int tree_find_mnt_ents(struct mnt_list *mnts, struct list_head *list, const char
 		return 0;
 
 	plen = strlen(path);
-	mlen = strlen(mnts->path);
+	mlen = strlen(mnts->mp);
 	if (mlen < plen)
 		return tree_find_mnt_ents(mnts->right, list, path);
 	else if (mlen > plen)
@@ -1370,7 +1400,7 @@ int tree_find_mnt_ents(struct mnt_list *mnts, struct list_head *list, const char
 
 		tree_find_mnt_ents(mnts->left, list, path);
 
-		if (!strcmp(mnts->path, path)) {
+		if (!strcmp(mnts->mp, path)) {
 			INIT_LIST_HEAD(&mnts->entries);
 			list_add(&mnts->entries, list);
 		}
@@ -1381,7 +1411,7 @@ int tree_find_mnt_ents(struct mnt_list *mnts, struct list_head *list, const char
 
 			this = list_entry(p, struct mnt_list, self);
 
-			if (!strcmp(this->path, path)) {
+			if (!strcmp(this->mp, path)) {
 				INIT_LIST_HEAD(&this->entries);
 				list_add(&this->entries, list);
 			}
@@ -1396,50 +1426,6 @@ int tree_find_mnt_ents(struct mnt_list *mnts, struct list_head *list, const char
 	return 0;
 }
 
-int tree_is_mounted(struct mnt_list *mnts, const char *path, unsigned int type)
-{
-	struct ioctl_ops *ops = get_ioctl_ops();
-	struct list_head *p;
-	struct list_head list;
-	int mounted = 0;
-
-	if (ops->ismountpoint)
-		return ioctl_is_mounted(_PROC_MOUNTS, path, type);
-
-	INIT_LIST_HEAD(&list);
-
-	if (!tree_find_mnt_ents(mnts, &list, path))
-		return 0;
-
-	list_for_each(p, &list) {
-		struct mnt_list *mptr;
-
-		mptr = list_entry(p, struct mnt_list, entries);
-
-		if (type) {
-			unsigned int autofs_fs;
-
-			autofs_fs = !strcmp(mptr->fs_type, "autofs");
-
-			if (type & MNTS_REAL) {
-				if (!autofs_fs) {
-					mounted = 1;
-					break;
-				}
-			} else if (type & MNTS_AUTOFS) {
-				if (autofs_fs) {
-					mounted = 1;
-					break;
-				}
-			} else {
-				mounted = 1;
-				break;
-			}
-		}
-	}
-	return mounted;
-}
-
 void set_tsd_user_vars(unsigned int logopt, uid_t uid, gid_t gid)
 {
 	struct thread_stdenv_vars *tsv;
@@ -1539,7 +1525,7 @@ void set_tsd_user_vars(unsigned int logopt, uid_t uid, gid_t gid)
 		status = getgrgid_r(gid, pgr, gr_tmp, tmplen, ppgr);
 		if (status != ERANGE)
 			break;
-		tmplen += grplen;
+		tmplen *= 2;
 
 		/* Don't tempt glibc to alloca() larger than is (likely)
 		 * available on the stack if limit-getgrgid-size is enabled.
@@ -1983,7 +1969,7 @@ void set_indirect_mount_tree_catatonic(struct autofs_point *ap)
 	struct mapent_cache *mc;
 	struct mapent *me;
 
-	if (!is_mounted(_PROC_MOUNTS, ap->path, MNTS_AUTOFS))
+	if (!is_mounted(ap->path, MNTS_AUTOFS))
 		return;
 
 	map = entry->maps;
@@ -2049,7 +2035,7 @@ int umount_ent(struct autofs_point *ap, const char *path)
 		 * so that we do not try to call rmdir_path on the
 		 * directory.
 		 */
-		if (!rv && is_mounted(_PATH_MOUNTED, path, MNTS_REAL)) {
+		if (!rv && is_mounted(path, MNTS_REAL)) {
 			crit(ap->logopt,
 			     "the umount binary reported that %s was "
 			     "unmounted, but there is still something "
@@ -2182,7 +2168,7 @@ int mount_multi_triggers(struct autofs_point *ap, struct mapent *me,
 		 */
 		if (ap->state == ST_READMAP && ap->flags & MOUNT_FLAG_REMOUNT) {
 			if (oe->ioctlfd != -1 ||
-			    is_mounted(_PROC_MOUNTS, oe->key, MNTS_REAL)) {
+			    is_mounted(oe->key, MNTS_REAL)) {
 				char oe_root[PATH_MAX + 1];
 				strcpy(oe_root, root);
 				strcat(oe_root, offset); 
@@ -2271,7 +2257,7 @@ int umount_multi_triggers(struct autofs_point *ap, struct mapent *me, char *root
 		left += umount_multi_triggers(ap, oe, root, oe_base);
 
 		if (oe->ioctlfd != -1 ||
-		    is_mounted(_PROC_MOUNTS, oe->key, MNTS_REAL)) {
+		    is_mounted(oe->key, MNTS_REAL)) {
 			left++;
 			continue;
 		}
@@ -2314,10 +2300,10 @@ int umount_multi_triggers(struct autofs_point *ap, struct mapent *me, char *root
 		 * delete the offsets from the cache and we need to put
 		 * the offset triggers back.
 		 */
-		if (is_mounted(_PATH_MOUNTED, root, MNTS_REAL)) {
+		if (is_mounted(root, MNTS_REAL)) {
 			info(ap->logopt, "unmounting dir = %s", root);
 			if (umount_ent(ap, root) &&
-			    is_mounted(_PATH_MOUNTED, root, MNTS_REAL)) {
+			    is_mounted(root, MNTS_REAL)) {
 				if (mount_multi_triggers(ap, me, root, strlen(root), "/") < 0)
 					warn(ap->logopt,
 					     "failed to remount offset triggers");
@@ -2417,9 +2403,9 @@ int clean_stale_multi_triggers(struct autofs_point *ap,
 		 * ESTALE errors when attempting list the directory.
 		 */
 		if (oe->ioctlfd != -1 ||
-		    is_mounted(_PROC_MOUNTS, oe->key, MNTS_REAL)) {
+		    is_mounted(oe->key, MNTS_REAL)) {
 			if (umount_ent(ap, oe->key) &&
-			    is_mounted(_PROC_MOUNTS, oe->key, MNTS_REAL)) {
+			    is_mounted(oe->key, MNTS_REAL)) {
 				debug(ap->logopt,
 				      "offset %s has active mount, invalidate",
 				      oe->key);
