@@ -43,6 +43,7 @@ const rpcvers_t rpcb_version = PMAPVERS;
 
 #include "mount.h"
 #include "rpc_subs.h"
+#include "replicated.h"
 #include "automount.h"
 
 /* #define STANDALONE */
@@ -182,7 +183,7 @@ static int rpc_do_create_client(struct sockaddr *addr, struct conn_info *info, i
 	in4_laddr.sin_addr.s_addr = htonl(INADDR_ANY);
 	slen = sizeof(struct sockaddr_in);
 
-	if (!info->client) {
+	if (!info->client && *fd == RPC_ANYSOCK) {
 		struct sockaddr *laddr;
 
 		*fd = open_sock(addr->sa_family, type, proto);
@@ -295,7 +296,7 @@ static int rpc_do_create_client(struct sockaddr *addr, struct conn_info *info, i
 	 * it would bind to a reserved port, which has been shown to
 	 * exhaust the reserved port range in some situations.
 	 */
-	if (!info->client) {
+	if (!info->client && *fd == RPC_ANYSOCK) {
 		*fd = open_sock(addr->sa_family, type, proto);
 		if (*fd < 0) {
 			ret = -errno;
@@ -388,6 +389,7 @@ static enum clnt_stat rpc_get_netid(const sa_family_t family,
 		}
 
 		*netid = nc_netid;
+		break;
 	}
 	endnetconfig(handle);
 	free(nc_proto);
@@ -682,7 +684,7 @@ static int create_client(struct conn_info *info, CLIENT **client)
 	}
 
 	memset(&hints, 0, sizeof(hints));
-	hints.ai_flags = AI_ADDRCONFIG;
+	hints.ai_flags = AI_ADDRCONFIG | AI_CANONNAME;
 	hints.ai_family = AF_UNSPEC;
 	if (info->proto == IPPROTO_UDP)
 		hints.ai_socktype = SOCK_DGRAM;
@@ -692,7 +694,8 @@ static int create_client(struct conn_info *info, CLIENT **client)
 	ret = getaddrinfo(info->host, NULL, &hints, &ai);
 	if (ret) {
 		error(LOGOPT_ANY,
-		      "hostname lookup failed: %s", gai_strerror(ret));
+		      "hostname lookup for %s failed: %s",
+		      info->host, gai_strerror(ret));
 		goto out_close;
 	}
 
@@ -1001,12 +1004,11 @@ int rpc_ping_proto(struct conn_info *info)
 }
 
 static int __rpc_ping(const char *host,
-		      unsigned long version, int proto,
+		      unsigned long version, int proto, int port,
 		      long seconds, long micros, unsigned int option)
 {
 	int status;
 	struct conn_info info;
-	struct pmap parms;
 
 	info.proto = proto;
 	info.host = host;
@@ -1023,52 +1025,67 @@ static int __rpc_ping(const char *host,
 
 	status = RPC_PING_FAIL;
 
-	parms.pm_prog = NFS_PROGRAM;
-	parms.pm_vers = version;
-	parms.pm_prot = info.proto;
-	parms.pm_port = 0;
 
-	status = rpc_portmap_getport(&info, &parms, &info.port);
-	if (status < 0)
-		return status;
+	if (port > 0)
+		info.port = port;
+	else {
+		struct pmap parms;
+
+		parms.pm_prog = NFS_PROGRAM;
+		parms.pm_vers = version;
+		parms.pm_prot = info.proto;
+		parms.pm_port = 0;
+		status = rpc_portmap_getport(&info, &parms, &info.port);
+		if (status < 0)
+			return status;
+	}
 
 	status = rpc_ping_proto(&info);
 
 	return status;
 }
 
-int rpc_ping(const char *host, long seconds, long micros, unsigned int option)
+int rpc_ping(const char *host, int port,
+	     unsigned int version, long seconds, long micros,
+	     unsigned int option)
 {
-	unsigned long vers4 = NFS4_VERSION;
-	unsigned long vers3 = NFS3_VERSION;
-	unsigned long vers2 = NFS2_VERSION;
-	int status;
+	int status = 0;
 
-	status = __rpc_ping(host, vers2, IPPROTO_UDP, seconds, micros, option);
-	if (status > 0)
-		return RPC_PING_V2 | RPC_PING_UDP;
+	if ((version & NFS2_REQUESTED) && (version & TCP_REQUESTED)) {
+		status = __rpc_ping(host, NFS2_VERSION,
+				    IPPROTO_TCP, port, seconds, micros, option);
+		if (status > 0)
+			return RPC_PING_V2 | RPC_PING_TCP;
+	}
 
-	status = __rpc_ping(host, vers3, IPPROTO_UDP, seconds, micros, option);
-	if (status > 0)
-		return RPC_PING_V3 | RPC_PING_UDP;
+	if ((version & NFS2_REQUESTED) && (version & UDP_REQUESTED)) {
+		status = __rpc_ping(host, NFS2_VERSION,
+				    IPPROTO_UDP, port, seconds, micros, option);
+		if (status > 0)
+			return RPC_PING_V2 | RPC_PING_UDP;
+	}
 
-	/* UDP isn't recommended for NFSv4, don't bother checking it.
-	status = __rpc_ping(host, vers4, IPPROTO_UDP, seconds, micros, option);
-	if (status > 0)
-		return RPC_PING_V4 | RPC_PING_UDP;
-	*/
+	if ((version & NFS3_REQUESTED) && (version & TCP_REQUESTED)) {
+		status = __rpc_ping(host, NFS3_VERSION,
+				    IPPROTO_TCP, port, seconds, micros, option);
+		if (status > 0)
+			return RPC_PING_V3 | RPC_PING_TCP;
+	}
 
-	status = __rpc_ping(host, vers2, IPPROTO_TCP, seconds, micros, option);
-	if (status > 0)
-		return RPC_PING_V2 | RPC_PING_TCP;
+	if ((version & NFS3_REQUESTED) && (version & UDP_REQUESTED)) {
+		status = __rpc_ping(host, NFS3_VERSION,
+				    IPPROTO_UDP, port, seconds, micros, option);
+		if (status > 0)
+			return RPC_PING_V3 | RPC_PING_UDP;
+	}
 
-	status = __rpc_ping(host, vers3, IPPROTO_TCP, seconds, micros, option);
-	if (status > 0)
-		return RPC_PING_V3 | RPC_PING_TCP;
-
-	status = __rpc_ping(host, vers4, IPPROTO_TCP, seconds, micros, option);
-	if (status > 0)
-		return RPC_PING_V4 | RPC_PING_TCP;
+	if (version & NFS4_REQUESTED) {
+		/* UDP isn't recommended for NFSv4, don't check it. */
+		status = __rpc_ping(host, NFS4_VERSION,
+				    IPPROTO_TCP, port, seconds, micros, option);
+		if (status > 0)
+			return RPC_PING_V4 | RPC_PING_TCP;
+	}
 
 	return status;
 }
@@ -1078,37 +1095,12 @@ double monotonic_elapsed(struct timespec start, struct timespec end)
 	double t1, t2;
 
 	t1 =  (double) start.tv_sec +
-		(double) (start.tv_nsec/(1000*1000*1000));
+		((double) start.tv_nsec/(1000*1000*1000));
 	t2 =  (double) end.tv_sec +
-		(double) (end.tv_nsec/(1000*1000*1000));
+		((double) end.tv_nsec/(1000*1000*1000));
 	return t2 - t1;
 }
 
-int rpc_time(const char *host,
-	     unsigned int ping_vers, unsigned int ping_proto,
-	     long seconds, long micros, unsigned int option, double *result)
-{
-	int status;
-	double taken;
-	struct timespec start, end;
-	int proto = (ping_proto & RPC_PING_UDP) ? IPPROTO_UDP : IPPROTO_TCP;
-	unsigned long vers = ping_vers;
-
-	clock_gettime(CLOCK_MONOTONIC, &start);
-	status = __rpc_ping(host, vers, proto, seconds, micros, option);
-	clock_gettime(CLOCK_MONOTONIC, &end);
-
-	if (status == RPC_PING_FAIL || status < 0)
-		return status;
-
-	taken = monotonic_elapsed(start, end);
-
-	if (result != NULL)
-		*result = taken;
-
-	return status;
-}
-
 static int rpc_get_exports_proto(struct conn_info *info, exports *exp)
 {
 	CLIENT *client;
@@ -1281,64 +1273,3 @@ const char *get_addr_string(struct sockaddr *sa, char *name, socklen_t len)
 
 	return inet_ntop(sa->sa_family, addr, name, len);
 }
-
-#if 0
-#include <stdio.h>
-
-int main(int argc, char **argv)
-{
-	int ret;
-	double res = 0.0;
-	exports exportlist, tmp;
-	groups grouplist;
-	int n, maxlen;
-
-/*
-	ret = rpc_ping("budgie", 10, 0, RPC_CLOSE_DEFAULT);
-	printf("ret = %d\n", ret);
-
-	res = 0.0;
-	ret = rpc_time("budgie", NFS2_VERSION, RPC_PING_TCP, 10, 0, RPC_CLOSE_DEFAULT, &res);
-	printf("v2 tcp ret = %d, res = %f\n", ret, res);
-
-	res = 0.0;
-	ret = rpc_time("budgie", NFS3_VERSION, RPC_PING_TCP, 10, 0, RPC_CLOSE_DEFAULT, &res);
-	printf("v3 tcp ret = %d, res = %f\n", ret, res);
-
-	res = 0.0;
-	ret = rpc_time("budgie", NFS2_VERSION, RPC_PING_UDP, 10, 0, RPC_CLOSE_DEFAULT, &res);
-	printf("v2 udp ret = %d, res = %f\n", ret, res);
-
-	res = 0.0;
-	ret = rpc_time("budgie", NFS3_VERSION, RPC_PING_UDP, 10, 0, RPC_CLOSE_DEFAULT, &res);
-	printf("v3 udp ret = %d, res = %f\n", ret, res);
-*/
-	exportlist = rpc_get_exports("budgie", 10, 0, RPC_CLOSE_NOLINGER);
-	exportlist = rpc_exports_prune(exportlist);
-
-	maxlen = 0;
-	for (tmp = exportlist; tmp; tmp = tmp->ex_next) {
-		if ((n = strlen(tmp->ex_dir)) > maxlen)
-			maxlen = n;
-	}
-
-	if (exportlist) {
-		while (exportlist) {
-			printf("%-*s ", maxlen, exportlist->ex_dir);
-			grouplist = exportlist->ex_groups;
-			if (grouplist) {
-				while (grouplist) {
-					printf("%s%s", grouplist->gr_name,
-						grouplist->gr_next ? "," : "");
-					grouplist = grouplist->gr_next;
-				}
-			}
-			printf("\n");
-			exportlist = exportlist->ex_next;
-		}
-	}
-	rpc_exports_free(exportlist);
-
-	exit(0);
-}
-#endif
