@@ -46,7 +46,8 @@ int lookup_version = AUTOFS_LOOKUP_VERSION;	/* Required by protocol */
 exports rpc_get_exports(const char *host, long seconds, long micros, unsigned int option);
 void rpc_exports_free(exports list);
 
-int lookup_init(const char *mapfmt, int argc, const char *const *argv, void **context)
+int lookup_init(const char *mapfmt,
+		int argc, const char *const *argv, void **context)
 {
 	struct lookup_context *ctxt;
 	char buf[MAX_ERR_BUF];
@@ -68,11 +69,27 @@ int lookup_init(const char *mapfmt, int argc, const char *const *argv, void **co
 		free(ctxt);
 		return 1;
 	}
+
 	*context = ctxt;
 
 	return 0;
 }
 
+int lookup_reinit(const char *mapfmt,
+		  int argc, const char *const *argv, void **context)
+{
+	struct lookup_context *ctxt = (struct lookup_context *) *context;
+	int ret;
+
+	mapfmt = MAPFMT_DEFAULT;
+
+	ret = reinit_parse(ctxt->parse, mapfmt, MODPREFIX, argc, argv);
+	if (ret)
+		return 1;
+
+	return 0;
+}
+
 int lookup_read_master(struct master *master, time_t age, void *context)
 {
 	return NSS_STATUS_UNKNOWN;
@@ -162,9 +179,9 @@ static int do_parse_mount(struct autofs_point *ap, struct map_source *source,
 	return NSS_STATUS_SUCCESS;
 }
 
-static int update_hosts_mounts(struct autofs_point *ap,
-			       struct map_source *source, time_t age,
-			       struct lookup_context *ctxt)
+static void update_hosts_mounts(struct autofs_point *ap,
+				struct map_source *source, time_t age,
+				struct lookup_context *ctxt)
 {
 	struct mapent_cache *mc;
 	struct mapent *me;
@@ -212,13 +229,14 @@ next:
 		ap->flags |= MOUNT_FLAG_REMOUNT;
 		ret = ctxt->parse->parse_mount(ap, me->key, strlen(me->key),
 					       me->mapent, ctxt->parse->context);
+		if (ret)
+			warn(ap->logopt, MODPREFIX
+			     "failed to parse mount %s", me->mapent);
 		ap->flags &= ~MOUNT_FLAG_REMOUNT;
 cont:
 		me = cache_lookup_next(mc, me);
 	}
 	pthread_cleanup_pop(1);
-
-	return NSS_STATUS_SUCCESS;
 }
 
 int lookup_read_map(struct autofs_point *ap, time_t age, void *context)
@@ -284,7 +302,7 @@ int lookup_mount(struct autofs_point *ap, const char *name, int name_len, void *
 	struct mapent *me;
 	char *mapent = NULL;
 	int mapent_len;
-	time_t now = time(NULL);
+	time_t now = monotonic_time(NULL);
 	int ret;
 
 	source = ap->entry->current;
@@ -296,27 +314,9 @@ int lookup_mount(struct autofs_point *ap, const char *name, int name_len, void *
 	/* Check if we recorded a mount fail for this key anywhere */
 	me = lookup_source_mapent(ap, name, LKP_DISTINCT);
 	if (me) {
-		if (me->status >= time(NULL)) {
-			cache_unlock(me->mc);
+		/* negative timeout has not passed, return fail */
+		if (cache_lookup_negative(me, name) == CHE_UNAVAIL)
 			return NSS_STATUS_NOTFOUND;
-		} else {
-			struct mapent_cache *smc = me->mc;
-			struct mapent *sme;
-
-			if (me->mapent)
-				cache_unlock(smc);
-			else {
-				cache_unlock(smc);
-				cache_writelock(smc);
-				sme = cache_lookup_distinct(smc, name);
-				/* Negative timeout expired for non-existent entry. */
-				if (sme && !sme->mapent) {
-					if (cache_pop_mapent(sme) == CHE_FAIL)
-						cache_delete(smc, name);
-				}
-				cache_unlock(smc);
-			}
-		}
 	}
 
 	cache_readlock(mc);
