@@ -46,10 +46,14 @@ int mount_mount(struct autofs_point *ap, const char *root, const char *name, int
 	char buf[MAX_ERR_BUF];
 	int err;
 	int len, status, existed = 1;
+	void (*mountlog)(unsigned int, const char*, ...) = &log_debug;
 
 	if (ap->flags & MOUNT_FLAG_REMOUNT)
 		return 0;
 
+	if (defaults_get_mount_verbose())
+		mountlog = &log_info;
+
 	/* Root offset of multi-mount */
 	len = strlen(root);
 	if (root[len - 1] == '/') {
@@ -80,15 +84,15 @@ int mount_mount(struct autofs_point *ap, const char *root, const char *name, int
 		existed = 0;
 
 	if (options && options[0]) {
-		debug(ap->logopt,
-		      MODPREFIX "calling mount -t %s -o %s %s %s",
-		      fstype, options, what, fullpath);
+		mountlog(ap->logopt,
+			 MODPREFIX "calling mount -t %s -o %s %s %s",
+			 fstype, options, what, fullpath);
 
 		err = spawn_mount(ap->logopt, "-t", fstype,
 				  "-o", options, what, fullpath, NULL);
 	} else {
-		debug(ap->logopt, MODPREFIX "calling mount -t %s %s %s",
-		      fstype, what, fullpath);
+		mountlog(ap->logopt, MODPREFIX "calling mount -t %s %s %s",
+			 fstype, what, fullpath);
 		err = spawn_mount(ap->logopt, "-t", fstype, what, fullpath, NULL);
 	}
 
