@@ -24,11 +24,11 @@
  *        - PROXIMITY_OTHER, host is on a network not directlty
  *          reachable through a local interface.
  *
- *   2) NFS version and protocol is selected by caclculating the largest
+ *   2) NFS version and protocol is selected by calculating the largest
  *      number of hosts supporting an NFS version and protocol that
  *      have the closest proximity. These hosts are added to the list
  *      in response time order. Hosts may have a corresponding weight
- *      which essentially increaes response time and so influences the
+ *      which essentially increases response time and so influences the
  *      host order.
  *
  *   3) Hosts at further proximity that support the selected NFS version
@@ -497,9 +497,6 @@ static int check_address_proto(unsigned logopt,
 	if (ipv6_requested) {
 		if (host->addr_len == INET_ADDRSTRLEN)
 			ret = 0;
-	} else {
-		if (host->addr_len == INET6_ADDRSTRLEN)
-			ret = 0;
 	}
 
 	if (!ret)
